@@ -2,19 +2,22 @@ import copy
 import json
 import secrets
 import time
+import csv
 from collections import OrderedDict
-from typing import Optional, Dict, Iterable, List, Union
+from typing import Optional, Dict, Iterable, List, Union, Set
+import dataclasses
 
 import click
 from box import Box, BoxList
 from tabulate import tabulate
 
 from montecarlodata.common.common import struct_match
-from montecarlodata.common.data import EventProperties, AWSArn, DcResourceProperties
+from montecarlodata.common.data import EventProperties, AWSArn, DcResourceProperties, ResourceProperties, BucketRow
 from montecarlodata.common.user import UserService
 from montecarlodata.config import Config
 from montecarlodata.errors import complain_and_abort, manage_errors, prompt_connection
-from montecarlodata.utils import AwsClientWrapper
+from montecarlodata.utils import AwsClientWrapper, GqlWrapper
+from montecarlodata.queries.onboarding import EventOnboardingDataQueries
 
 
 class CloudResourceService:
@@ -39,12 +42,13 @@ class CloudResourceService:
 
     def __init__(self, config: Config, user_service: Optional[UserService] = None,
                  aws_wrapper: Optional[AwsClientWrapper] = None, aws_profile_override: Optional[str] = None,
-                 aws_region_override: Optional[str] = None):
+                 aws_region_override: Optional[str] = None, request_wrapper: Optional[GqlWrapper] = None):
         self._abort_on_error = True
         self._profile = aws_profile_override or config.aws_profile
         self._region = aws_region_override or config.aws_region
+        self._request_wrapper = request_wrapper or GqlWrapper(config)
 
-        self._user_service = user_service or UserService(config=config)
+        self._user_service = user_service or UserService(config=config, request_wrapper=self._request_wrapper)
         self._aws_wrapper = aws_wrapper or AwsClientWrapper(profile_name=self._profile, region_name=self._region)
 
     @manage_errors
@@ -192,19 +196,20 @@ class CloudResourceService:
         click.echo('[Gathering collector and resource details]')
         collector = self.get_and_validate_active_collector(dc_id=dc_id)
         event_details = self._initialize_events(collector_props=collector, **kwargs)
-        click.echo('[Creating or updating topic properties]')
         self._create_or_update_event_topic(event_details=event_details, skip_prompts=auto_yes)
-        click.echo('[Updating event queue properties]')
         self._update_event_q(event_details=event_details, skip_prompts=auto_yes)
-        click.echo('[Creating topic subscription]')
         self._create_topic_subscription(event_details=event_details, skip_prompts=auto_yes)
-        click.echo('[Creating event notification]')
         self._create_event_notification(event_details=event_details, skip_prompts=auto_yes)
         click.echo('\nProcess complete. Have a nice day!')
 
-    def _initialize_events(self, bucket_name: str, event_type: str, collector_props: Box,
+    def _initialize_events(self, event_type: str, collector_props: Box,
                            collector_aws_profile: Optional[str] = None,
-                           bucket_aws_profile: Optional[str] = None, **kwargs) -> EventProperties:
+                           bucket_aws_profile: Optional[str] = None, 
+                           buckets_filename: Optional[str] = None,
+                           bucket_name: Optional[str] = None, 
+                           prefix: Optional[str] = None,
+                           suffix: Optional[str] = None,
+                           **kwargs) -> EventProperties:
         """
         Collect (or lookup) any relevant event related properties and initialize any clients.
 
@@ -214,85 +219,298 @@ class CloudResourceService:
         collector_profile = collector_aws_profile or self._profile
         cust_resources_profile = bucket_aws_profile or self._profile
 
-        # Create a temp client to lookup the region of the bucket (reduces number of prompts).
-        temp_client = AwsClientWrapper(profile_name=cust_resources_profile, region_name=collector_arn.region)
-        cust_resources_region = temp_client.get_bucket_location(bucket_name=bucket_name)
-
         # Create a client for collector resources and any customer resources.
         # This is necessary as the collector resources might be in a different account/region than customer resources.
         collection_client = AwsClientWrapper(profile_name=collector_profile, region_name=collector_arn.region)
-        cust_resources_client = AwsClientWrapper(profile_name=cust_resources_profile, region_name=cust_resources_region)
 
         collector_props.outputs = collection_client.get_stack_outputs(stack_id=collector_props.stackArn)
         event_queue_arn = self._get_relevant_q(event_type, collector_props.outputs)
 
-        return EventProperties(
+        resource_properties = self._get_resource_properties(
+            cust_resources_profile=cust_resources_profile,
+            buckets_filename=buckets_filename,
             bucket_name=bucket_name,
+            prefix=prefix,
+            suffix=suffix,
+            **kwargs
+        )
+
+        return EventProperties(
             event_type=event_type,
             event_queue_arn=event_queue_arn,
             collection_region=collector_arn.region,
-            cust_resources_region=cust_resources_region,
             collection_account_id=collector_arn.account,
-            cust_account_id=cust_resources_client.get_caller_identity(),
             collection_client=collection_client,
-            cust_resources_client=cust_resources_client,
+            resource_properties=resource_properties,
+        )
+
+    def create_event_topic(self, dc_id: Optional[str] = None, auto_yes: Optional[bool] = False, **kwargs):
+        collector = self.get_and_validate_active_collector(dc_id=dc_id)
+        event_details = self._populate_event_details_from_saved_config(collector, **kwargs)
+        self._create_or_update_event_topic(event_details=event_details, skip_prompts=auto_yes)
+
+    def create_bucket_side_event_infrastructure(self, dc_id: Optional[str] = None, auto_yes: Optional[bool] = False, **kwargs):
+        collector = self.get_and_validate_active_collector(dc_id=dc_id)
+        event_details = self._populate_event_details_from_saved_config(collector, **kwargs)
+        self._create_topic_subscription(event_details=event_details, skip_prompts=auto_yes)
+        self._create_event_notification(event_details=event_details, skip_prompts=auto_yes)
+
+    def update_event_queue(self, dc_id: Optional[str] = None, auto_yes: Optional[bool] = False, **kwargs):
+        collector = self.get_and_validate_active_collector(dc_id=dc_id)
+        event_details = self._populate_event_details_from_saved_config(collector, collection_client=True, **kwargs)
+        self._update_event_q(event_details=event_details, skip_prompts=auto_yes)
+
+    def save_initial_event_properties(self, dc_id: Optional[str] = None, **kwargs):
+        collector = self.get_and_validate_active_collector(dc_id=dc_id)
+        event_details = self._populate_event_details_from_saved_config(collector, collection_client=True, **kwargs)
+        self._save_event_properties(event_details)
+
+    def delete_event_properties(self):
+        self._delete_event_properties()
+
+    def _get_clients_for_buckets(self, buckets, cust_resources_profile):
+        region_to_client = {}
+        bucket_to_client = {}
+        temp_client = AwsClientWrapper(profile_name=cust_resources_profile)
+        for bucket in buckets:
+            bucket_name = bucket['bucket_name']
+            region = temp_client.get_bucket_location(bucket_name=bucket_name)
+            if region not in region_to_client:
+                region_to_client[region] = AwsClientWrapper(profile_name=cust_resources_profile, region_name=region)
+
+            bucket_to_client[bucket_name] = region_to_client[region]
+
+        return bucket_to_client
+
+
+    def parse_buckets_file(self, buckets_filename: str) -> List[BucketRow]:
+        buckets_file = csv.DictReader(open(buckets_filename))
+        return [
+            BucketRow(
+                bucket_name=row['bucket_name'],
+                prefix=row.get('prefix'),
+                suffix=row.get('suffix')
+            )
+            for row in buckets_file
+        ]
+
+
+    def _get_resource_properties(self,
+            cust_resources_profile: Optional[str] = None,
+            buckets_filename: Optional[str] = None,
+            bucket_name: Optional[str] = None,
+            prefix: Optional[str] = None,
+            suffix: Optional[str] = None,
             **kwargs
+        ) -> List[ResourceProperties]:
+
+        if not buckets_filename and not bucket_name:
+            return []
+
+        if bucket_name:
+            args = {}
+            if prefix:
+                args['prefix'] = prefix
+            if suffix:
+                args['suffix'] = suffix
+            buckets = [{"bucket_name": bucket_name, **args}]
+
+        if buckets_filename:
+            bucket_rows = self.parse_buckets_file(buckets_filename)
+            buckets = [dataclasses.asdict(bucket_row) for bucket_row in bucket_rows]
+
+        bucket_clients = self._get_clients_for_buckets(buckets, cust_resources_profile)
+        
+        return [
+            ResourceProperties(
+                bucket_name=bucket['bucket_name'],
+                client=bucket_clients[bucket['bucket_name']],
+                region=bucket_clients[bucket['bucket_name']].region,
+                account_id=bucket_clients[bucket['bucket_name']].get_caller_identity(),
+                prefix=bucket.get('prefix'),
+                suffix=bucket.get('suffix'),
+                **kwargs
+        ) for bucket in buckets]
+
+    def _populate_event_details_from_saved_config(
+        self, 
+        collector_props: Box, 
+        event_type: Optional[str] = None,
+        collector_aws_profile: Optional[str] = None,
+        bucket_aws_profile: Optional[str] = None,
+        collection_client: bool = False,
+        buckets_filename: Optional[str] = None,
+        bucket_name: Optional[str] = None,
+        prefix: Optional[str] = None,
+        suffix: Optional[str] = None,
+        **kwargs
+    ):
+        collector_arn = AWSArn(id=collector_props.stackArn)
+        collector_profile = collector_aws_profile or self._profile
+        cust_resources_profile = bucket_aws_profile or self._profile
+
+        response = self._request_wrapper.make_request_v2(
+            query=EventOnboardingDataQueries.get.query,
+            operation=EventOnboardingDataQueries.get.operation
+        )
+
+        config = {'resource_properties': []}
+        if response.data:
+            config = json.loads(response.data.config)
+            if not collection_client:
+                bucket_clients = self._get_clients_for_buckets(config['resource_properties'], cust_resources_profile)
+            config['resource_properties'] = [
+                ResourceProperties(
+                    bucket_name=bucket['bucket_name'],
+                    client=bucket_clients[bucket['bucket_name']] if not collection_client else None,
+                    region=bucket.get('region'),
+                    account_id=bucket.get('account_id'),
+                    prefix=bucket.get('prefix'),
+                    suffix=bucket.get('suffix'),
+                    topic_arn=bucket.get('topic_arn'),
+                    topic_name=bucket.get('topic_name'),
+                ) for bucket in config['resource_properties']
+            ]
+
+        if not event_type and config.get('event_type'):
+            event_type = config['event_type']
+
+        resource_properties = self._get_resource_properties(
+            cust_resources_profile=cust_resources_profile,
+            buckets_filename=buckets_filename,
+            bucket_name=bucket_name,
+            prefix=prefix,
+            suffix=suffix,
+            **kwargs
+        )
+
+        if collection_client:
+            aws_client = AwsClientWrapper(profile_name=collector_profile, region_name=collector_arn.region)
+            collector_props.outputs = aws_client.get_stack_outputs(stack_id=collector_props.stackArn)
+            aws_options = dict(
+                collection_account_id=aws_client.get_caller_identity(),
+                collection_client=aws_client,
+                event_queue_arn=config.get('event_queue_arn') or self._get_relevant_q(event_type, collector_props.outputs)
+            )
+
+        else:
+            aws_options = dict(
+                collection_client=None,
+                collection_account_id=config.get('collection_account_id'),
+            )
+
+        overrides = {
+            'event_type': event_type,
+            'resource_properties': resource_properties,
+            'collection_region': collector_arn.region,
+        }
+
+        event_args = {
+            **config,
+            **aws_options,
+            **{key: value for key, value in overrides.items() if value}
+        }
+
+        return EventProperties(**event_args,)
+
+    def _save_event_properties(self, event_properties: EventProperties):
+        config = event_properties.serializable()
+        self._request_wrapper.make_request_v2(
+            query=EventOnboardingDataQueries.save.query,
+            operation=EventOnboardingDataQueries.save.operation,
+            variables = {"config": json.dumps(config)}
+        )
+
+    def _delete_event_properties(self):
+        self._request_wrapper.make_request_v2(
+            query=EventOnboardingDataQueries.delete.query,
+            operation=EventOnboardingDataQueries.delete.operation
         )
 
     def _create_or_update_event_topic(self, event_details: EventProperties, skip_prompts: Optional[bool] = False) -> None:
         """
         Creates topic (if necessary) and/or updates policy to allow access from bucket (put) & event queue (subscribe)
         """
-        new_statements = []
-        b_topic_policy_id = self._MCD_TOPIC_POLICY_SID_PATTERN.format(event_details.event_type, event_details.bucket_name)
+        click.echo('[Creating or updating topic properties]')
         q_topic_policy_id = self._MCD_TOPIC_POLICY_SID_PATTERN.format(event_details.event_type, event_details.event_queue_name)
 
-        if event_details.topic_arn is None:
+        region_to_resources = {}
+        for resource in event_details.resource_properties:
+            region = resource.region
+            if region not in region_to_resources:
+                region_to_resources[region] = []
+
+            region_to_resources[region].append(resource)
+
+        for region, resources in region_to_resources.items():
+            new_statements = []
+            update_statements = {}
+            region_bucket_arns = [resource.bucket_arn for resource in resources]
+
             # Create an event topic if an MC one does not already exist in the region for the queue type
-            name = self._MCD_TOPIC_NAME_PATTERN.format(event_details.cust_resources_region, event_details.event_type)
-            prompt = f'Create topic \'{name}\' in \'{event_details.cust_resources_region}\' ' \
-                     f'({event_details.cust_account_id})? This is an idempotent operation.'
-            prompt_connection(message=prompt, skip_prompt=skip_prompts)
-            event_details.topic_arn = event_details.cust_resources_client.create_sns_topic(name=name)
-        event_details.topic_name = self._validate_topic_arn(event_details=event_details).resource
+            topic_arn = resources[0].topic_arn
+            if topic_arn is None:
+                name = self._MCD_TOPIC_NAME_PATTERN.format(region, event_details.event_type)
+                prompt = f'Create topic \'{name}\' in \'{region}\' ' \
+                         f'({resources[0].account_id})? This is an idempotent operation.'
+                prompt_connection(message=prompt, skip_prompt=skip_prompts)
+                topic_arn = resources[0].client.create_sns_topic(name=name)
 
-        # Update policy if necessary
-        topic_attributes = event_details.cust_resources_client.get_topic_attributes(arn=event_details.topic_arn)
-        if not self._is_preexisting_topic_policy(topic_policy_id=b_topic_policy_id, topic_attributes=topic_attributes):
-            new_statements.append(self._generate_sns_bucket_policy(sid=b_topic_policy_id, event_details=event_details))
-        if not self._is_preexisting_topic_policy(topic_policy_id=q_topic_policy_id, topic_attributes=topic_attributes):
-            new_statements.append(self._generate_sns_q_policy(sid=q_topic_policy_id, event_details=event_details))
+            for resource in resources:
+                resource.topic_arn = topic_arn
+                resource.topic_name = self._validate_topic_arn(resource_details=resource).resource
 
-        new_statement_count = len(new_statements)
-        if new_statement_count > 0:
-            click.echo(f'Adding statements (x{new_statement_count}) to \'{event_details.topic_name}\' -')
-            click.echo(json.dumps(new_statements, indent=4))
-            prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
-            event_details.cust_resources_client.set_topic_attributes(
-                arn=event_details.topic_arn,
-                name='Policy',
-                value=self._extend_topic_policy(topic_attributes=topic_attributes, new_statements=new_statements)
-            )
-        else:
-            click.echo(f'No policy changes required for \'{event_details.topic_name}\'')
+            topic_attributes = resources[0].client.get_topic_attributes(arn=resource.topic_arn)
+
+            region_bucket_topic_policy_id = self._MCD_TOPIC_POLICY_SID_PATTERN.format(event_details.event_type, resources[0].topic_name)
+
+            existing_bucket_policy = self._get_preexisting_topic_policy(topic_policy_id=region_bucket_topic_policy_id, topic_attributes=topic_attributes)
+
+            if not existing_bucket_policy:
+                new_statements.append(self._generate_sns_bucket_policy(sid=region_bucket_topic_policy_id, topic_arn=topic_arn, bucket_arns=region_bucket_arns))
+            else:
+                update_statements[region_bucket_topic_policy_id] = self._extend_sns_bucket_policy(existing_policy=existing_bucket_policy, bucket_arns=region_bucket_arns)
+
+            if not self._is_preexisting_topic_policy(topic_policy_id=q_topic_policy_id, topic_attributes=topic_attributes):
+                new_statements.append(self._generate_sns_q_policy(sid=q_topic_policy_id, event_details=event_details, topic_arn=topic_arn))
+
+            # Update policy if necessary
+
+            new_statement_count = len(new_statements) + len(update_statements)
+            if new_statement_count > 0:
+                click.echo(f'Adding statements (x{new_statement_count}) to \'{resources[0].topic_name}\' -')
+                click.echo(json.dumps([*new_statements, *update_statements.values()], indent=4))
+                prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
+                resources[0].client.set_topic_attributes(
+                    arn=topic_arn,
+                    name='Policy',
+                    value=self._extend_topic_policy(topic_attributes=topic_attributes, new_statements=new_statements, update_statements=update_statements)
+                )
+            else:
+                click.echo(f'No policy changes required for \'{resources[0].topic_name}\'')
+
+        self._save_event_properties(event_details)
 
     def _update_event_q(self, event_details: EventProperties, skip_prompts: Optional[bool] = False) -> None:
         """
         Create or update queue policy with topic
         """
+        click.echo('[Updating event queue properties]')
         attributes = event_details.collection_client.get_q_attributes(
             name=event_details.event_queue_name,
             attributes=['Policy']
         ) or {}
 
+        topic_arns = list(OrderedDict.fromkeys([resource.topic_arn for resource in event_details.resource_properties]).keys())
+
         if not attributes.get('Attributes', {}).get('Policy'):
             # No policy set (e.g. is a new q). Create
-            new_policy = self._generate_q_policy(event_details=event_details)
+            new_policy = self._generate_q_policy(event_details=event_details, topic_arns=topic_arns)
             click.echo(f'No existing policy found. Adding -\n{json.dumps(new_policy, indent=4)}')
         else:
             # Update existing if necessary
             existing_policy = Box(json.loads(attributes.Attributes.Policy))
-            new_policy = self._extend_q_policy(existing_policy=existing_policy, event_details=event_details)
+            new_policy = self._extend_q_policy(existing_policy=existing_policy, event_details=event_details, topic_arns=topic_arns)
             if struct_match(existing_policy, new_policy):
                 click.echo(f'Policy for \'{event_details.event_queue_name}\' requires no changes')
                 return
@@ -304,55 +522,68 @@ class CloudResourceService:
             attributes=dict(Policy=json.dumps(new_policy))
         )
 
+        self._save_event_properties(event_details)
+
     def _create_event_notification(self, event_details: EventProperties, skip_prompts: Optional[bool] = False) -> None:
         """
         Creates event notification if one does not already exist for the topic
         """
-        setting_id = self._MCD_EVENT_ID_PATTERN.format(
-            AWSArn(id=event_details.topic_arn).resource.lower(),
-            f'-{event_details.prefix}' if event_details.prefix else '',
-            f'-{event_details.suffix}' if event_details.suffix else ''
-        )
+        click.echo('[Creating event notification]')
+        for resource in event_details.resource_properties:
+            setting_id = self._MCD_EVENT_ID_PATTERN.format(
+                AWSArn(id=resource.topic_arn).resource.lower(),
+                f'-{resource.prefix}' if resource.prefix else '',
+                f'-{resource.suffix}' if resource.suffix else ''
+            )
 
-        # Get existing notification settings for bucket as AWS does not support inserting (only a put operation)
-        existing_config = event_details.cust_resources_client.get_bucket_event_config(name=event_details.bucket_name)
-        existing_config.pop('ResponseMetadata', None)
-        message = 'Updating existing notifications into -' if existing_config else 'Creating notification -'
+            # Get existing notification settings for bucket as AWS does not support inserting (only a put operation)
+            existing_config = resource.client.get_bucket_event_config(name=resource.bucket_name)
+            existing_config.pop('ResponseMetadata', None)
+            message = 'Updating existing notifications into -' if existing_config else 'Creating notification -'
 
-        new_config = self._merge_event_notifications(
-            setting_id=setting_id,
-            existing_config=existing_config,
-            event_details=event_details
-        )
+            new_config = self._merge_event_notifications(
+                setting_id=setting_id,
+                existing_config=existing_config,
+                resource_details=resource
+            )
 
-        if struct_match(existing_config, new_config):
-            click.echo(f'Notifications for \'{event_details.bucket_name}\' require no changes')
-            return
+            if struct_match(existing_config, new_config):
+                click.echo(f'Notifications for \'{resource.bucket_name}\' require no changes')
+                continue
 
-        click.echo(f'{message}\n{json.dumps(new_config, indent=4)}')
-        prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
-        event_details.cust_resources_client.set_bucket_event_config(
-            name=event_details.bucket_name,
-            notification_config=new_config
-        )
+            click.echo(f'{message}\n{json.dumps(new_config, indent=4)}')
+            prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
+            resource.client.set_bucket_event_config(
+                name=resource.bucket_name,
+                notification_config=new_config
+            )
+
+        self._save_event_properties(event_details)
 
     def _create_topic_subscription(self, event_details: EventProperties, skip_prompts: Optional[bool] = False) -> None:
         """
         Create topic subscription for endpoint if one for the endpoint already does not exist
         """
-        subscriptions = list(event_details.cust_resources_client.list_topic_subscriptions(arn=event_details.topic_arn) or [])
-        if subscriptions:
-            for subscription in [sub for sublist in subscriptions for sub in sublist]:
-                if subscription.get('Endpoint') == event_details.event_queue_arn:
-                    click.echo(f'Subscription for \'{event_details.event_queue_name}\' already exists')
-                    return
-        click.echo(f'Creating subscription for \'{event_details.event_queue_name}\' on \'{event_details.topic_name}\'')
-        prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
-        event_details.cust_resources_client.subscribe_to_topic(
-            arn=event_details.topic_arn,
-            endpoint=event_details.event_queue_arn,
-            attributes=dict(RawMessageDelivery='true')
-        )
+        click.echo('[Creating topic subscription]')
+        for resource in event_details.resource_properties:
+            should_continue = False
+            subscriptions = list(resource.client.list_topic_subscriptions(arn=resource.topic_arn) or [])
+            if subscriptions:
+                for subscription in [sub for sublist in subscriptions for sub in sublist]:
+                    if subscription.get('Endpoint') == event_details.event_queue_arn:
+                        click.echo(f'Subscription for \'{event_details.event_queue_name}\' already exists')
+                        should_continue = True
+            if should_continue:
+                continue
+            click.echo(f'Creating subscription for \'{event_details.event_queue_name}\' on \'{resource.topic_name}\'')
+            prompt_connection(message='Please confirm', skip_prompt=skip_prompts)
+            resource.client.subscribe_to_topic(
+                arn=resource.topic_arn,
+                endpoint=event_details.event_queue_arn,
+                attributes=dict(RawMessageDelivery='true')
+            )
+
+        self._save_event_properties(event_details)
 
     def _get_relevant_q(self, event_type: str, outputs: Union[Box, BoxList]) -> str:
         """
@@ -369,41 +600,73 @@ class CloudResourceService:
         complain_and_abort('Failed to find the relevant event queue. Are events enabled for this collector?')
 
     @staticmethod
-    def _validate_topic_arn(event_details: EventProperties) -> AWSArn:
+    def _validate_topic_arn(resource_details: ResourceProperties) -> AWSArn:
         """
         Sanity check ARN (e.g. if provided) is valid (format + region)
         """
         try:
-            arn = AWSArn(id=event_details.topic_arn)
+            arn = AWSArn(id=resource_details.topic_arn)
         except IndexError:
-            complain_and_abort(f'SNS topic ARN \'{event_details.topic_arn}\' is not a valid.')
+            complain_and_abort(f'SNS topic ARN \'{resource_details.topic_arn}\' is not a valid.')
         else:
-            if event_details.cust_resources_region != arn.region:
+            if resource_details.region != arn.region:
                 complain_and_abort(f'Topic region ({arn.region}) does not match bucket '
-                                   f'region ({event_details.cust_resources_region})')
+                                   f'region ({resource_details.region})')
             return arn
 
     @staticmethod
-    def _is_preexisting_topic_policy(topic_policy_id: str, topic_attributes: Box) -> bool:
+    def _get_preexisting_topic_policy(topic_policy_id: str, topic_attributes: Box) -> Optional[Box]:
+        for statement in Box(json.loads(topic_attributes.Attributes.Policy)).Statement:
+            if statement.get('Sid') and statement.Sid == topic_policy_id:
+                return statement
+        return None
+
+    def _is_preexisting_topic_policy(self, topic_policy_id: str, topic_attributes: Box) -> bool:
         """
         Check if a policy for this bucket already exists. Returns true if it does.
         """
-        for statement in Box(json.loads(topic_attributes.Attributes.Policy)).Statement:
-            if statement.get('Sid') and statement.Sid == topic_policy_id:
-                return True
-        return False
+        return bool(self._get_preexisting_topic_policy(topic_policy_id, topic_attributes))
 
     @staticmethod
-    def _extend_topic_policy(topic_attributes: Box, new_statements: List[Dict]) -> str:
+    def _extend_topic_policy(topic_attributes: Box, new_statements: List[Dict], update_statements: Dict) -> str:
         """
         Merge new statements with existing
         """
         policy = Box(json.loads(topic_attributes.Attributes.Policy))
-        policy.Statement.extend(new_statements)
+        new_policy_statement = []
+        for statement in policy.Statement:
+            if statement.Sid in update_statements:
+                new_policy_statement.append(update_statements[statement.Sid])
+            else:
+                new_policy_statement.append(statement)
+        new_policy_statement.extend(new_statements)
+        policy.Statement = new_policy_statement
         return json.dumps(policy)
 
     @staticmethod
-    def _extend_q_policy(existing_policy: Box, event_details: EventProperties) -> Dict:
+    def _extend_sns_bucket_policy(existing_policy: Box, bucket_arns: List[str]) -> Dict:
+        """
+        Merge new statements with existing
+        """
+        new_policy = Box(copy.deepcopy(existing_policy))
+        source_arn = new_policy.get('Condition', {}).get('StringEquals', {}).get('aws:SourceArn', [])
+        if source_arn:
+            # Source ARN can be a list or string :/
+            if not isinstance(source_arn, List):
+                source_arn = [source_arn]
+
+            source_arn = list(OrderedDict.fromkeys([*source_arn, *bucket_arns]).keys())
+
+            new_policy.Condition.StringEquals['aws:SourceArn'] = source_arn
+            if len(source_arn) == 1:
+                new_policy.Condition.StringEquals['aws:SourceArn'] = source_arn[0]
+            return new_policy
+        
+        # Existing policy was create outside of tooling / different from steps outlined in documentation
+        complain_and_abort('Queue policy exists, but not in a supported format')
+
+    @staticmethod
+    def _extend_q_policy(existing_policy: Box, event_details: EventProperties, topic_arns: Optional[Iterable[str]]) -> Dict:
         """
         Merge new statements with existing
         """
@@ -413,10 +676,11 @@ class CloudResourceService:
                 source_arn = statement.get('Condition', {}).get('ArnLike', {}).get('aws:SourceArn')
                 if source_arn:
                     # Source ARN can be a list or string :/
+                    list_topic_arns = list(topic_arns)
                     if isinstance(source_arn, List):
-                        source_arn.append(event_details.topic_arn)
+                        source_arn.extend(list_topic_arns)
                     else:
-                        source_arn = [source_arn, event_details.topic_arn]
+                        source_arn = [source_arn, *list_topic_arns]
                     # Remove duplicates, while still retaining order/type for computing diff.
                     statement.Condition.ArnLike['aws:SourceArn'] = list(OrderedDict.fromkeys(source_arn).keys())
                     if len(statement.Condition.ArnLike['aws:SourceArn']) == 1:
@@ -425,7 +689,7 @@ class CloudResourceService:
         # Existing policy was create outside of tooling / different from steps outlined in documentation
         complain_and_abort('Queue policy exists, but not in a supported format')
 
-    def _merge_event_notifications(self, setting_id: str, existing_config: Dict, event_details: EventProperties) -> Dict:
+    def _merge_event_notifications(self, setting_id: str, existing_config: Dict, resource_details: ResourceProperties) -> Dict:
         """
         Merge new config with existing based on if and how topic config was set
         """
@@ -438,7 +702,7 @@ class CloudResourceService:
             config['TopicConfigurations'].append(
                 self._generate_bucket_to_topic_config(
                     setting_id=setting_id,
-                    event_details=event_details
+                    resource_details=resource_details
                 )
             )
         else:
@@ -446,29 +710,29 @@ class CloudResourceService:
             config['TopicConfigurations'] = [
                 self._generate_bucket_to_topic_config(
                     setting_id=setting_id,
-                    event_details=event_details
+                    resource_details=resource_details
                 )
             ]
         return config
 
     @staticmethod
-    def _generate_bucket_to_topic_config(setting_id: str, event_details: EventProperties) -> Dict:
+    def _generate_bucket_to_topic_config(setting_id: str, resource_details: ResourceProperties) -> Dict:
         return dict(
             Id=setting_id,
-            TopicArn=event_details.topic_arn,
+            TopicArn=resource_details.topic_arn,
             Events=['s3:ObjectCreated:*', 's3:ObjectRemoved:*'],
             Filter=dict(
                 Key=dict(
                     FilterRules=[
-                        dict(Name='Prefix', Value=event_details.prefix or ''),
-                        dict(Name='Suffix', Value=event_details.suffix or '')
+                        dict(Name='Prefix', Value=resource_details.prefix or ''),
+                        dict(Name='Suffix', Value=resource_details.suffix or '')
                     ]
                 )
             )
         )
 
     @staticmethod
-    def _generate_sns_bucket_policy(sid: str, event_details: EventProperties) -> Dict:
+    def _generate_sns_bucket_policy(sid: str, topic_arn: str, bucket_arns: List[str]) -> Dict:
         return {
             'Sid': sid,
             'Effect': 'Allow',
@@ -476,16 +740,16 @@ class CloudResourceService:
                 'AWS': '*'
             },
             'Action': 'SNS:Publish',
-            'Resource': event_details.topic_arn,
+            'Resource': topic_arn,
             'Condition': {
                 'StringEquals': {
-                    'aws:SourceArn': event_details.bucket_arn
+                    'aws:SourceArn': bucket_arns
                 }
             }
         }
 
     @staticmethod
-    def _generate_sns_q_policy(sid: str, event_details: EventProperties) -> Dict:
+    def _generate_sns_q_policy(sid: str, event_details: EventProperties, topic_arn: str) -> Dict:
         return {
             'Sid': sid,
             'Effect': 'Allow',
@@ -493,11 +757,11 @@ class CloudResourceService:
                 'AWS': event_details.collection_account_id
             },
             'Action': 'sns:Subscribe',
-            'Resource': event_details.topic_arn
+            'Resource': topic_arn
         }
 
     @staticmethod
-    def _generate_q_policy(event_details: EventProperties):
+    def _generate_q_policy(event_details: EventProperties, topic_arns: Optional[Iterable[str]]):
         return {
             'Version': '2008-10-17',
             'Statement': list(filter(None, [
@@ -520,12 +784,10 @@ class CloudResourceService:
                     'Resource': event_details.event_queue_arn,
                     'Condition': {
                         'ArnLike': {
-                            'aws:SourceArn': [
-                                event_details.topic_arn
-                            ]
+                            'aws:SourceArn': list(topic_arns)
                         }
                     }
-                } if event_details.topic_arn else None
+                } if topic_arns else None
             ]))
         }
 
