@@ -23,7 +23,7 @@ from montecarlodata.utils import GqlWrapper
 
 class MonteCarloConfigService:
     def __init__(self, config: Config, request_wrapper: Optional[GqlWrapper] = None, project_dir: str = None):
-        request_wrapper = request_wrapper or GqlWrapper(mcd_id=config.mcd_id, mcd_token=config.mcd_token)
+        request_wrapper = request_wrapper or GqlWrapper(config)
         self.client = MonteCarloConfigTemplateClient(request_wrapper)
 
         if not project_dir:
