@@ -70,6 +70,9 @@ WAREHOUSE_CREATE_OPTIONS = [
 WAREHOUSE_SELECT_OPTIONS = [
     click.option('--name', help='Friendly name of the warehouse which the connection will belong to.', required=False)
 ]
+WAREHOUSE_OPTIONAL_CREATE_OPTIONS = [
+    click.option('--create-warehouse', help='Create a new warehouse with this connection', type=click.BOOL, default=True, required=False)
+]
 S3_OPTIONS = [
     click.option('--bucket', help='S3 Bucket where query logs are contained.', required=True),
     click.option('--prefix', help='Path to query logs.', required=True),
@@ -193,6 +196,7 @@ def integrations():
 @click.option('--catalog', help='Presto catalog name. For using multiple hive clusters with Presto. '
                                 'Uses \'hive\' if not specified.', required=False)
 @add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(WAREHOUSE_OPTIONAL_CREATE_OPTIONS)
 @add_common_options(DATABASE_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
@@ -352,6 +356,7 @@ def add_spark_http_mode(ctx, user, name, **kwargs):
 @integrations.command(help=f'Setup a Spark integration for Databricks. {SQL_VERBIAGE}.')
 @click.pass_obj
 @add_common_options(DATABRICKS_OPTIONS)
+@add_common_options(WAREHOUSE_SELECT_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_spark_databricks(ctx, name, **kwargs):
@@ -384,9 +389,9 @@ def add_databricks_metastore(ctx, name, **kwargs):
 @integrations.command(help='Setup a Databricks Delta integration. For metadata queries on delta tables when using an external metastore in databricks.')
 @click.pass_obj
 @add_common_options(DATABRICKS_OPTIONS)
+@add_common_options(WAREHOUSE_SELECT_OPTIONS)
 @add_common_options(DATABRICKS_DATA_COLLECTOR_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_databricks_delta(ctx, name, **kwargs):
     """
@@ -604,11 +609,35 @@ def add_looker_git(ctx, **kwargs):
     ReportsOnboardingService(config=ctx['config']).onboard_looker_git(**kwargs)
 
 
+@integrations.command(help=f'Setup a Power BI integration. {BI_VERBIAGE}.')
+@click.pass_obj
+@click.option('--tenant-id', help='The tenant ID from the Azure Active Directory.', required=True)
+@click.option('--auth-mode', help='Authentication Mode. We support service principal and master user two auth types',
+              required=True, type=click.Choice(['SERVICE_PRINCIPAL', 'MASTER_USER']))
+@click.option('--client-id', help='App registration application ID for accessing Power BI.',
+              required=True, cls=AdvancedOptions)
+@click.option('--client-secret', help='Secret for the application to access the Power BI. Set only when auth-mode is '
+                                      'SERVICE_PRINCIPAL', required=False, cls=AdvancedOptions,
+              mutually_exclusive_options=['username', 'password'], prompt_if_requested=True)
+@click.option('--username', help='Username for accessing the Power BI. Set only when auth-mode is MASTER_USER.',
+              cls=AdvancedOptions, required=False, mutually_exclusive_options=['client-secret'])
+@click.option('--password', help='Password for accessing the Power BI. Set only when auth-mode is MASTER_USER.',
+              cls=AdvancedOptions, required=False, mutually_exclusive_options=['client-secret'],
+              prompt_if_requested=True)
+@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
+def add_power_bi(ctx, **kwargs):
+    """
+    Onboard a Power BI connection
+    """
+    ReportsOnboardingService(config=ctx['config']).onboard_power_bi(**kwargs)
+
+
 @integrations.command(help='Setup a dbt Cloud integration.')
 @click.pass_obj
 @click.option('--dbt-cloud-api-token', 'dbt_cloud_api_token', help=f'dbt Cloud API token. {PASSWORD_VERBIAGE}.',
               required=True, cls=AdvancedOptions, prompt_if_requested=True)
 @click.option('--dbt-cloud-account-id', 'dbt_cloud_account_id', help='dbt Cloud Account ID.', required=True)
+@click.option('--dbt-cloud-base-url', 'dbt_cloud_base_url', help='dbt Cloud Base URL.', required=False)
 @add_common_options(WAREHOUSE_SELECT_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 def add_dbt_cloud(ctx, name, **kwargs):
@@ -704,6 +733,7 @@ def add_events(ctx, **kwargs):
     CloudResourceService(config=ctx['config'], aws_wrapper=None).add_events(**kwargs)
 
 
+@integrations.command(help='Setup Event Topic for S3 event notifications in a lake.')
 @click.pass_obj
 @add_common_options(S3_BUCKET_OPTIONS)
 @add_common_options(EVENT_TYPE_OPTIONS)
