@@ -179,9 +179,10 @@ class BaseOnboardingService:
         """
         warehouse_type = onboarding_config.warehouse_type or DATA_LAKE_WAREHOUSE_TYPE  # default to data-lake
         connection_type = onboarding_config.connection_type
+        create_warehouse = onboarding_config.create_warehouse
 
         # Main connections require the creation of a new warehouse.
-        if connection_type in MAIN_CONNECTION_TYPES or warehouse_type != DATA_LAKE_WAREHOUSE_TYPE:
+        if (connection_type in MAIN_CONNECTION_TYPES or warehouse_type != DATA_LAKE_WAREHOUSE_TYPE) and create_warehouse:
             return {'createWarehouseType': warehouse_type}
 
         warehouses = self._user_service.warehouses
