@@ -20,6 +20,7 @@ EXPECTED_TEST_TABLEAU_RESPONSE_FIELD = 'testTableauCredentials'
 EXPECTED_LOOKER_METADATA_RESPONSE_FIELD = 'testLookerCredentials'
 EXPECTED_LOOKER_GIT_SSH_RESPONSE_FILED = 'testLookerGitSshCredentials'
 EXPECTED_LOOKER_GIT_CLONE_RESPONSE_FIELD = 'testLookerGitCloneCredentials'
+EXPECTED_POWER_BI_RESPONSE_FIELD = 'testPowerBiCredentials'
 EXPECTED_DBT_CLOUD_RESPONSE_FIELD = 'testDbtCloudCredentials'
 EXPECTED_ADD_BI_RESPONSE_FIELD = 'addBiConnection'
 
@@ -43,6 +44,7 @@ LOOKER_GIT_CLONE_CONNECTION_TYPE = 'looker-git-clone'
 DBT_CLOUD_CONNECTION_TYPE = 'dbt-cloud'
 DATABRICKS_METASTORE_CONNECTION_TYPE = 'databricks-metastore'
 DATABRICKS_DELTA_CONNECTION_TYPE = 'databricks-delta'
+POWER_BI_CONNECTION_TYPE = 'power-bi'
 
 METASTORE_CONNECTION_TYPES = (
     HIVE_MYSQL_CONNECTION_TYPE, GLUE_CONNECTION_TYPE, DATABRICKS_METASTORE_CONNECTION_TYPE
