@@ -15,8 +15,8 @@ mutation generateCollectorTemplate($region:String, $dcId:UUID) {
 """
 
 ADD_COLLECTOR_RECORD = """
-mutation createCollectorRecord {
-  createCollectorRecord {
+mutation createCollectorRecord($templateProvider:String, $templateVariant:String) {
+  createCollectorRecord(templateProvider:$templateProvider, templateVariant:$templateVariant) {
     dc {
       uuid
     }
