@@ -37,6 +37,8 @@ _SAMPLE_COLLECTORS_RESPONSE = [
         'stackArn': 'aws-loki',
         'active': True,
         'customerAwsAccountId': 'test',
+        'templateProvider': 'cloudformation',
+        'templateVariant': 'dionysus',
         'templateVersion': '42',
         'codeVersion': '2042',
         'lastUpdated': None
@@ -45,6 +47,8 @@ _SAMPLE_COLLECTORS_RESPONSE = [
         'uuid': '5678',
         'stackArn': 'aws-thor',
         'active': True,
+        'templateProvider': 'terraform',
+        'templateVariant': 'foo',
         'templateVersion': '42',
         'codeVersion': '2042',
         'lastUpdated': None
@@ -53,6 +57,8 @@ _SAMPLE_COLLECTORS_RESPONSE = [
         'uuid': '0912',
         'stackArn': 'aws-odin',
         'active': False,
+        'templateProvider': None,
+        'templateVariant': None,
         'templateVersion': '42',
         'codeVersion': '2042',
         'lastUpdated': None
@@ -203,19 +209,21 @@ class CollectorManagementTest(TestCase):
         self._user_service_mock.collectors = _SAMPLE_COLLECTORS_RESPONSE
 
         self._service.echo_collectors(table_format='plain')
-        click_mock.echo.assert_called_once_with('AWS Stack ARN      ID    Version  Last updated    Active\n'
-                                                'aws-loki         1234       2042  -               True\n'
-                                                'aws-thor         5678       2042  -               True\n'
-                                                'aws-odin         0912       2042  -               False')
+        click_mock.echo.assert_called_once_with(
+            'AWS Stack ARN      ID    Version  Template                    Last updated    Active\n'
+            'aws-loki         1234       2042  cloudformation:dionysus:42  -               True\n'
+            'aws-thor         5678       2042  terraform:foo:42            -               True\n'
+            'aws-odin         0912       2042  cloudformation:July-2019    -               False')
 
     @patch('montecarlodata.collector.management.click')
     def test_echo_collectors_with_only_active(self, click_mock):
         self._user_service_mock.collectors = _SAMPLE_COLLECTORS_RESPONSE
 
         self._service.echo_collectors(table_format='plain', active_only=True)
-        click_mock.echo.assert_called_once_with('AWS Stack ARN      ID    Version  Last updated    Active\n'
-                                                'aws-loki         1234       2042  -               True\n'
-                                                'aws-thor         5678       2042  -               True')
+        click_mock.echo.assert_called_once_with(
+            'AWS Stack ARN      ID    Version  Template                    Last updated    Active\n'
+            'aws-loki         1234       2042  cloudformation:dionysus:42  -               True\n'
+            'aws-thor         5678       2042  terraform:foo:42            -               True')
 
     @patch.object(CollectorManagementService, 'launch_quick_create_link')
     @patch('montecarlodata.collector.management.prompt_connection')
